@@ -38,7 +38,8 @@ static bool canCreateSubCollection( const Collection &collection )
     return false;
   }
 
-  if ( !collection.contentMimeTypes().contains( Collection::mimeType() ) ) {
+  if ( !collection.contentMimeTypes().contains( Collection::mimeType() ) &&
+       !collection.contentMimeTypes().contains( Collection::virtualMimeType() ) ) {
     return false;
   }
 
@@ -52,7 +53,8 @@ static inline bool canContainItems( const Collection &collection )
   }
 
   if ( ( collection.contentMimeTypes().count() == 1 ) &&
-       ( collection.contentMimeTypes().first() == Collection::mimeType() ) ) {
+       ( ( collection.contentMimeTypes().first() == Collection::mimeType() ) ||
+         ( collection.contentMimeTypes().first() == Collection::virtualMimeType() ) ) ) {
     return false;
   }
 
