@@ -30,7 +30,7 @@
 #include "session_p.h"
 
 #include <kdebug.h>
-#include <KLocale>
+#include <KLocalizedString>
 
 #include <QtCore/QStringList>
 #include <QtCore/QTimer>
@@ -43,7 +43,7 @@ class Akonadi::ItemFetchJobPrivate : public JobPrivate
     ItemFetchJobPrivate( ItemFetchJob *parent )
       : JobPrivate( parent ),
         mEmitTimer( 0 ),
-	mValuePool( 0 )
+        mValuePool( 0 )
     {
       mCollection = Collection::root();
     }
