@@ -28,7 +28,7 @@
 #include "xdgbasedirs_p.h"
 
 #include <kdebug.h>
-#include <klocale.h>
+#include <klocalizedstring.h>
 
 #include <QCoreApplication>
 #include <QtCore/QDir>
@@ -195,8 +195,7 @@ void SessionPrivate::dataReceived()
       // handle login response
       if ( parser->tag() == QByteArray( "0" ) ) {
         if ( parser->data().startsWith( "OK" ) ) { //krazy:exclude=strings
-          connected = true;
-          startNext();
+          writeData("1 CAPABILITY (NOTIFY 2)");
         } else {
           kWarning() << "Unable to login to Akonadi server:" << parser->data();
           socket->close();
@@ -204,6 +203,16 @@ void SessionPrivate::dataReceived()
         }
       }
 
+      // handle capability response
+      if ( parser->tag() == QByteArray("1") ) {
+        if ( parser->data().startsWith("OK") ) {
+          connected = true;
+          startNext();
+        } else {
+          kDebug() << "Unhandled server capability response:" << parser->data();
+        }
+      }
+
       // send login command
       if ( parser->tag() == "*" && parser->data().startsWith( "OK Akonadi" ) ) {
         const int pos = parser->data().indexOf( "[PROTOCOL" );
@@ -369,7 +378,7 @@ void SessionPrivate::init( const QByteArray &id )
   }
 
   connected = false;
-  theNextTag = 1;
+  theNextTag = 2;
   jobRunning = false;
 
   if ( ServerManager::state() == ServerManager::NotRunning )
