@@ -134,6 +134,14 @@ class KCALCORE_EXPORT Incidence
     */
     virtual Incidence *clone() const = 0;
 
+    /**
+      Returns a unique identifier for a specific instance of an incidence.
+
+      Due to the recurrence-id, the uid is not unique for a KCalCore::Incidence.
+      @since 4.11
+    */
+    QString instanceIdentifier() const;
+
     /**
       Set readonly state of incidence.
 
@@ -143,8 +151,7 @@ class KCALCORE_EXPORT Incidence
     void setReadOnly( bool readonly );
 
     /**
-      @copydoc
-      IncidenceBase::setLastModified().
+      @copydoc IncidenceBase::setLastModified().
     */
     void setLastModified( const KDateTime &lm );
 
@@ -160,15 +167,14 @@ class KCALCORE_EXPORT Incidence
 
     /**
       Get the localOnly status.
+      @return true if Local only, false otherwise.
 
       @see setLocalOnly()
-      @return True if Local only, false otherwise
     */
     bool localOnly() const;
 
     /**
-      @copydoc
-      IncidenceBase::setAllDay().
+      @copydoc IncidenceBase::setAllDay().
     */
     void setAllDay( bool allDay );
 
@@ -216,8 +222,7 @@ class KCALCORE_EXPORT Incidence
     virtual void setDtStart( const KDateTime &dt );
 
     /**
-      @copydoc
-      IncidenceBase::shiftTimes()
+      @copydoc IncidenceBase::shiftTimes()
     */
     virtual void shiftTimes( const KDateTime::Spec &oldSpec,
                              const KDateTime::Spec &newSpec );
@@ -430,26 +435,22 @@ class KCALCORE_EXPORT Incidence
     void clearRecurrence();
 
     /**
-      @copydoc
-      Recurrence::recurs()
+      @copydoc Recurrence::recurs()
     */
     bool recurs() const;
 
     /**
-      @copydoc
-      Recurrence::recurrenceType()
+      @copydoc Recurrence::recurrenceType()
     */
     ushort recurrenceType() const;
 
     /**
-      @copydoc
-      Recurrence::recursOn()
+      @copydoc Recurrence::recursOn()
     */
     virtual bool recursOn( const QDate &date, const KDateTime::Spec &timeSpec ) const;
 
     /**
-      @copydoc
-      Recurrence::recursAt()
+      @copydoc Recurrence::recursAt()
     */
     bool recursAt( const KDateTime &dt ) const;
 
@@ -683,12 +684,14 @@ class KCALCORE_EXPORT Incidence
 
     /**
       Returns true if the incidence has recurrenceId, otherwise return false.
-      @see setHasRecurrenceID(), setRecurrenceId(KDateTime)
+      @see setRecurrenceId(KDateTime)
     */
     bool hasRecurrenceId() const;
 
     /**
       Set the incidences recurrenceId.
+      This field indicates that this is an exception to a recurring incidence.
+      The uid of this incidence MUST be the same as the one of the recurring main incidence.
       @param recurrenceId is the incidence recurrenceId to set
       @see recurrenceId().
     */
@@ -701,6 +704,23 @@ class KCALCORE_EXPORT Incidence
     */
     KDateTime recurrenceId() const;
 
+    /**
+      Set to true if the exception also applies to all future occurrences.
+      This option is only relevant if the incidence has a recurrenceId set.
+      @param thisAndFuture value
+      @see thisAndFuture(), setRecurrenceId()
+      @since 4.11
+    */
+    void setThisAndFuture( bool thisAndFuture );
+
+    /**
+      Returns true if the exception also applies to all future occurrences.
+      @return incidences thisAndFuture value
+      @see setThisAndFuture()
+      @since 4.11
+    */
+    bool thisAndFuture() const;
+
 // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 // %%%%%  Alarm-related methods
 // %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -802,7 +822,7 @@ class KCALCORE_EXPORT Incidence
      * Returns true if the incidence type supports groupware communication.
      * @since 4.10
      */ //TODO_KDE5: make pure virtual
-    /*virtual*/ bool supportsGroupwareCommunication() const/* = 0*/;
+    bool supportsGroupwareCommunication() const;
 
   protected:
 
@@ -820,8 +840,7 @@ class KCALCORE_EXPORT Incidence
     virtual bool equals( const IncidenceBase &incidence ) const;
 
     /**
-      @copydoc
-      IncidenceBase::assign()
+      @copydoc IncidenceBase::assign()
     */
     virtual IncidenceBase &assign( const IncidenceBase &other );
 
@@ -849,7 +868,9 @@ inline uint qHash( const QSharedPointer<KCalCore::Incidence> &key )
 }
 //@endcond
 
+//@cond PRIVATE
 Q_DECLARE_TYPEINFO( KCalCore::Incidence::Ptr, Q_MOVABLE_TYPE );
 Q_DECLARE_METATYPE( KCalCore::Incidence * )
+//@endcond
 
 #endif
