@@ -20,13 +20,12 @@
 #include "sessionthread_p.h"
 
 #include <QtCore/QDebug>
-#include <QtCore/QTimer>
+#include <QtCore/QThread>
 
 #include <KDE/KDebug>
 
 #include "imapstreamparser.h"
 #include "message_p.h"
-#include "session.h"
 
 using namespace KIMAP;
 
@@ -35,40 +34,47 @@ Q_DECLARE_METATYPE( KSslErrorUiData )
 static const int _kimap_socketErrorTypeId = qRegisterMetaType<KTcpSocket::Error>();
 static const int _kimap_sslErrorUiData = qRegisterMetaType<KSslErrorUiData>();
 
-SessionThread::SessionThread( const QString &hostName, quint16 port, Session *parent )
-  : QThread(), m_hostName( hostName ), m_port( port ),
-    m_session( parent ), m_socket( 0 ), m_stream( 0 ), m_mutex( QMutex::Recursive ),
+SessionThread::SessionThread( const QString &hostName, quint16 port )
+  : QObject(), m_hostName( hostName ), m_port( port ),
+    m_socket( 0 ), m_stream( 0 ), m_mutex(),
     m_encryptedMode( false ),
     triedSslVersions( 0 ), doSslFallback( false )
 {
-  // Yeah, sounds weird, but QThread object is linked to the parent
-  // thread not to itself, and I'm too lazy to introduce yet another
-  // internal QObject
-  moveToThread( this );
+  // Just like the Qt docs now recommend, for event-driven threads:
+  // don't derive from QThread, create one directly and move the object to it.
+  QThread* thread = new QThread();
+  moveToThread( thread );
+  thread->start();
+  QMetaObject::invokeMethod( this, "threadInit" );
 }
 
 SessionThread::~SessionThread()
 {
-  // don't call quit() directly, this will deadlock in wait() if exec() hasn't run yet
-  QMetaObject::invokeMethod( this, "quit" );
-  if ( !wait( 10 * 1000 ) ) {
+  QMetaObject::invokeMethod( this, "threadQuit" );
+  if ( !thread()->wait( 10 * 1000 ) ) {
     kWarning() << "Session thread refuses to die, killing harder...";
-    terminate();
+    thread()->terminate();
     // Make sure to wait until it's done, otherwise it can crash when the pthread callback is called
-    wait();
+    thread()->wait();
   }
+  delete thread();
 }
 
+// Called in primary thread
 void SessionThread::sendData( const QByteArray &payload )
 {
   QMutexLocker locker( &m_mutex );
 
   m_dataQueue.enqueue( payload );
-  QTimer::singleShot( 0, this, SLOT(writeDataQueue()) );
+  QMetaObject::invokeMethod( this, "writeDataQueue" );
 }
 
+// Called in secondary thread
 void SessionThread::writeDataQueue()
 {
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_socket )
+    return;
   QMutexLocker locker( &m_mutex );
 
   while ( !m_dataQueue.isEmpty() ) {
@@ -76,11 +82,11 @@ void SessionThread::writeDataQueue()
   }
 }
 
+// Called in secondary thread
 void SessionThread::readMessage()
 {
-  QMutexLocker locker( &m_mutex );
-
-  if ( m_stream->availableDataSize() == 0 ) {
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_stream || m_stream->availableDataSize() == 0 ) {
     return;
   }
 
@@ -124,26 +130,33 @@ void SessionThread::readMessage()
   }
 
   if ( m_stream->availableDataSize() > 1 ) {
-    QTimer::singleShot( 0, this, SLOT(readMessage()) );
+    QMetaObject::invokeMethod( this, "readMessage", Qt::QueuedConnection );
   }
 
 }
 
+// Called in main thread
 void SessionThread::closeSocket()
 {
-  QTimer::singleShot( 0, this, SLOT(doCloseSocket()) );
+  QMetaObject::invokeMethod( this, "doCloseSocket", Qt::QueuedConnection );
 }
 
+// Called in secondary thread
 void SessionThread::doCloseSocket()
 {
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_socket )
+    return;
   m_encryptedMode = false;
   m_socket->close();
 }
 
+// Called in secondary thread
 void SessionThread::reconnect()
 {
-  QMutexLocker locker( &m_mutex );
-  kDebug() << this << "state=" << m_socket->state();
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( m_socket == 0 ) // threadQuit already called
+    return;
   if ( m_socket->state() != SessionSocket::ConnectedState &&
        m_socket->state() != SessionSocket::ConnectingState ) {
     if ( m_encryptedMode ) {
@@ -154,43 +167,57 @@ void SessionThread::reconnect()
   }
 }
 
-void SessionThread::run()
+// Called in secondary thread
+void SessionThread::threadInit()
 {
+  Q_ASSERT( QThread::currentThread() == thread() );
   m_socket = new SessionSocket;
   m_stream = new ImapStreamParser( m_socket );
   connect( m_socket, SIGNAL(readyRead()),
            this, SLOT(readMessage()), Qt::QueuedConnection );
 
-  // Delay the call to socketDisconnected so that it finishes disconnecting before we call reconnect()
+  // Delay the call to slotSocketDisconnected so that it finishes disconnecting before we call reconnect()
   connect( m_socket, SIGNAL(disconnected()),
-           this, SLOT(socketDisconnected()), Qt::QueuedConnection );
+           this, SLOT(slotSocketDisconnected()), Qt::QueuedConnection );
   connect( m_socket, SIGNAL(connected()),
-           m_session, SLOT(socketConnected()) );
+           this, SIGNAL(socketConnected()) );
   connect( m_socket, SIGNAL(error(KTcpSocket::Error)),
-           this, SLOT(socketError()) );
+           this, SLOT(socketError(KTcpSocket::Error)) );
   connect( m_socket, SIGNAL(bytesWritten(qint64)),
-           m_session, SLOT(socketActivity()) );
+           this, SIGNAL(socketActivity()) );
   if ( m_socket->metaObject()->indexOfSignal( "encryptedBytesWritten(qint64)" ) > -1 ) {
       connect( m_socket, SIGNAL(encryptedBytesWritten(qint64)), // needs kdelibs > 4.8
-               m_session, SLOT(socketActivity()) );
+               this, SIGNAL(socketActivity()) );
   }
   connect( m_socket, SIGNAL(readyRead()),
-           m_session, SLOT(socketActivity()) );
+           this, SIGNAL(socketActivity()) );
 
-  connect( this, SIGNAL(responseReceived(KIMAP::Message)),
-           m_session, SLOT(responseReceived(KIMAP::Message)) );
-
-  QTimer::singleShot( 0, this, SLOT(reconnect()) );
-  exec();
-
-  delete m_stream;
-  delete m_socket;
+  QMetaObject::invokeMethod(this, "reconnect", Qt::QueuedConnection);
 }
 
-void SessionThread::startSsl(const KTcpSocket::SslVersion &version)
+// Called in secondary thread
+void SessionThread::threadQuit()
 {
-  QMutexLocker locker( &m_mutex );
+  Q_ASSERT( QThread::currentThread() == thread() );
+  delete m_stream;
+  m_stream = 0;
+  delete m_socket;
+  m_socket = 0;
+  thread()->quit();
+}
 
+// Called in primary thread
+void SessionThread::startSsl( KTcpSocket::SslVersion version )
+{
+  QMetaObject::invokeMethod( this, "doStartSsl", Q_ARG(KTcpSocket::SslVersion, version) );
+}
+
+// Called in secondary thread (via invokeMethod)
+void SessionThread::doStartSsl( KTcpSocket::SslVersion version )
+{
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_socket )
+    return;
   if ( version == KTcpSocket::AnySslVersion ) {
     doSslFallback = true;
     if ( m_socket->advertisedSslVersion() == KTcpSocket::UnknownSslVersion ) {
@@ -215,29 +242,37 @@ void SessionThread::startSsl(const KTcpSocket::SslVersion &version)
   m_socket->startClientEncryption();
 }
 
-void SessionThread::socketDisconnected()
+// Called in secondary thread
+void SessionThread::slotSocketDisconnected()
 {
+  Q_ASSERT( QThread::currentThread() == thread() );
   if ( doSslFallback ) {
     reconnect();
   } else {
-    QMetaObject::invokeMethod( m_session, "socketDisconnected" );
+    emit socketDisconnected();
   }
 }
 
-void SessionThread::socketError()
+// Called in secondary thread
+void SessionThread::socketError(KTcpSocket::Error error)
 {
-  QMutexLocker locker( &m_mutex );
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_socket )
+    return;
+  Q_UNUSED( error ); // can be used for debugging
   if ( doSslFallback ) {
-    locker.unlock(); // disconnectFromHost() ends up calling reconnect()
     m_socket->disconnectFromHost();
   } else {
-    QMetaObject::invokeMethod( m_session, "socketError" );
+    emit socketError();
   }
 }
 
+// Called in secondary thread
 void SessionThread::sslConnected()
 {
-  QMutexLocker locker( &m_mutex );
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_socket )
+    return;
   KSslCipher cipher = m_socket->sessionCipher();
 
   if ( m_socket->sslErrors().count() > 0 ||
@@ -260,7 +295,15 @@ void SessionThread::sslConnected()
 
 void SessionThread::sslErrorHandlerResponse(bool response)
 {
-  QMutexLocker locker( &m_mutex );
+  QMetaObject::invokeMethod(this, "doSslErrorHandlerResponse", Q_ARG(bool, response));
+}
+
+// Called in secondary thread (via invokeMethod)
+void SessionThread::doSslErrorHandlerResponse(bool response)
+{
+  Q_ASSERT( QThread::currentThread() == thread() );
+  if ( !m_socket )
+    return;
   if ( response ) {
     m_encryptedMode = true;
     emit encryptionNegotiationResult( true, m_socket->negotiatedSslVersion() );
@@ -275,4 +318,3 @@ void SessionThread::sslErrorHandlerResponse(bool response)
 }
 
 #include "moc_sessionthread_p.cpp"
-
