@@ -29,7 +29,7 @@
 #include <kmime/kmime_util.h>
 
 #include <KDebug>
-#include <KLocale>
+#include <KLocalizedString>
 #include <KUrl>
 
 #include <QtCore/QRegExp>
@@ -410,7 +410,6 @@ EmailParseResult KPIMUtils::isValidAddress( const QString &aStr )
               return MissingLocalPart;
             } else if ( index == strlen-1 ) {
               return MissingDomainPart;
-              break;
             }
           } else if ( inQuotedString ) {
             --atCount;
